# DO NOT EDIT! This file is automatically generated
import datetime
import enum
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import Address, CreatedBy, GeoJson, LastModifiedBy, LocalizedString
    from ._review import ReviewRatingStatistics
    from ._type import (
        CustomFields,
        CustomFieldsDraft,
        FieldContainer,
        TypeResourceIdentifier,
    )
__all__ = [
    "Channel",
    "ChannelAddRolesAction",
    "ChannelChangeDescriptionAction",
    "ChannelChangeKeyAction",
    "ChannelChangeNameAction",
    "ChannelDraft",
    "ChannelPagedQueryResponse",
    "ChannelReference",
    "ChannelRemoveRolesAction",
    "ChannelResourceIdentifier",
    "ChannelRoleEnum",
    "ChannelSetAddressAction",
    "ChannelSetCustomFieldAction",
    "ChannelSetCustomTypeAction",
    "ChannelSetGeoLocationAction",
    "ChannelSetRolesAction",
    "ChannelUpdate",
    "ChannelUpdateAction",
]


class Channel(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: :class:`str`
    key: str
    #: List of :class:`commercetools.types.ChannelRoleEnum`
    roles: typing.List["ChannelRoleEnum"]
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.Address`
    address: typing.Optional["Address"]
    #: Optional :class:`commercetools.types.ReviewRatingStatistics` `(Named` ``reviewRatingStatistics`` `in Commercetools)`
    review_rating_statistics: typing.Optional["ReviewRatingStatistics"]
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`commercetools.types.GeoJson` `(Named` ``geoLocation`` `in Commercetools)`
    geo_location: typing.Optional["GeoJson"]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        key: str,
        roles: typing.List["ChannelRoleEnum"],
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        name: typing.Optional["LocalizedString"] = None,
        description: typing.Optional["LocalizedString"] = None,
        address: typing.Optional["Address"] = None,
        review_rating_statistics: typing.Optional["ReviewRatingStatistics"] = None,
        custom: typing.Optional["CustomFields"] = None,
        geo_location: typing.Optional["GeoJson"] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.key = key
        self.roles = roles
        self.name = name
        self.description = description
        self.address = address
        self.review_rating_statistics = review_rating_statistics
        self.custom = custom
        self.geo_location = geo_location
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "Channel(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, key=%r, roles=%r, name=%r, description=%r, address=%r, review_rating_statistics=%r, custom=%r, geo_location=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.key,
                self.roles,
                self.name,
                self.description,
                self.address,
                self.review_rating_statistics,
                self.custom,
                self.geo_location,
            )
        )


class ChannelDraft(_BaseType):
    #: :class:`str`
    key: str
    #: Optional list of :class:`commercetools.types.ChannelRoleEnum`
    roles: typing.Optional[typing.List["ChannelRoleEnum"]]
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.Address`
    address: typing.Optional["Address"]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`commercetools.types.GeoJson` `(Named` ``geoLocation`` `in Commercetools)`
    geo_location: typing.Optional["GeoJson"]

    def __init__(
        self,
        *,
        key: str,
        roles: typing.Optional[typing.List["ChannelRoleEnum"]] = None,
        name: typing.Optional["LocalizedString"] = None,
        description: typing.Optional["LocalizedString"] = None,
        address: typing.Optional["Address"] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None,
        geo_location: typing.Optional["GeoJson"] = None
    ) -> None:
        self.key = key
        self.roles = roles
        self.name = name
        self.description = description
        self.address = address
        self.custom = custom
        self.geo_location = geo_location
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ChannelDraft(key=%r, roles=%r, name=%r, description=%r, address=%r, custom=%r, geo_location=%r)"
            % (
                self.key,
                self.roles,
                self.name,
                self.description,
                self.address,
                self.custom,
                self.geo_location,
            )
        )


class ChannelPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.Channel`
    results: typing.Sequence["Channel"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["Channel"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ChannelPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class ChannelReference(Reference):
    #: Optional :class:`commercetools.types.Channel`
    obj: typing.Optional["Channel"]

    def __init__(self, *, id: str, obj: typing.Optional["Channel"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.CHANNEL, id=id)

    def __repr__(self) -> str:
        return "ChannelReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class ChannelResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.CHANNEL, id=id, key=key)

    def __repr__(self) -> str:
        return "ChannelResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class ChannelRoleEnum(enum.Enum):
    INVENTORY_SUPPLY = "InventorySupply"
    PRODUCT_DISTRIBUTION = "ProductDistribution"
    ORDER_EXPORT = "OrderExport"
    ORDER_IMPORT = "OrderImport"
    PRIMARY = "Primary"


class ChannelUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "ChannelUpdate(version=%r, actions=%r)" % (self.version, self.actions)


class ChannelUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "ChannelUpdateAction(action=%r)" % (self.action,)


class ChannelAddRolesAction(ChannelUpdateAction):
    #: List of :class:`commercetools.types.ChannelRoleEnum`
    roles: typing.List["ChannelRoleEnum"]

    def __init__(self, *, roles: typing.List["ChannelRoleEnum"]) -> None:
        self.roles = roles
        super().__init__(action="addRoles")

    def __repr__(self) -> str:
        return "ChannelAddRolesAction(action=%r, roles=%r)" % (self.action, self.roles)


class ChannelChangeDescriptionAction(ChannelUpdateAction):
    #: :class:`commercetools.types.LocalizedString`
    description: "LocalizedString"

    def __init__(self, *, description: "LocalizedString") -> None:
        self.description = description
        super().__init__(action="changeDescription")

    def __repr__(self) -> str:
        return "ChannelChangeDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class ChannelChangeKeyAction(ChannelUpdateAction):
    #: :class:`str`
    key: str

    def __init__(self, *, key: str) -> None:
        self.key = key
        super().__init__(action="changeKey")

    def __repr__(self) -> str:
        return "ChannelChangeKeyAction(action=%r, key=%r)" % (self.action, self.key)


class ChannelChangeNameAction(ChannelUpdateAction):
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"

    def __init__(self, *, name: "LocalizedString") -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "ChannelChangeNameAction(action=%r, name=%r)" % (self.action, self.name)


class ChannelRemoveRolesAction(ChannelUpdateAction):
    #: List of :class:`commercetools.types.ChannelRoleEnum`
    roles: typing.List["ChannelRoleEnum"]

    def __init__(self, *, roles: typing.List["ChannelRoleEnum"]) -> None:
        self.roles = roles
        super().__init__(action="removeRoles")

    def __repr__(self) -> str:
        return "ChannelRemoveRolesAction(action=%r, roles=%r)" % (
            self.action,
            self.roles,
        )


class ChannelSetAddressAction(ChannelUpdateAction):
    #: Optional :class:`commercetools.types.Address`
    address: typing.Optional["Address"]

    def __init__(self, *, address: typing.Optional["Address"] = None) -> None:
        self.address = address
        super().__init__(action="setAddress")

    def __repr__(self) -> str:
        return "ChannelSetAddressAction(action=%r, address=%r)" % (
            self.action,
            self.address,
        )


class ChannelSetCustomFieldAction(ChannelUpdateAction):
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(self, *, name: str, value: typing.Optional[typing.Any] = None) -> None:
        self.name = name
        self.value = value
        super().__init__(action="setCustomField")

    def __repr__(self) -> str:
        return "ChannelSetCustomFieldAction(action=%r, name=%r, value=%r)" % (
            self.action,
            self.name,
            self.value,
        )


class ChannelSetCustomTypeAction(ChannelUpdateAction):
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.type = type
        self.fields = fields
        super().__init__(action="setCustomType")

    def __repr__(self) -> str:
        return "ChannelSetCustomTypeAction(action=%r, type=%r, fields=%r)" % (
            self.action,
            self.type,
            self.fields,
        )


class ChannelSetGeoLocationAction(ChannelUpdateAction):
    #: Optional :class:`commercetools.types.GeoJson` `(Named` ``geoLocation`` `in Commercetools)`
    geo_location: typing.Optional["GeoJson"]

    def __init__(self, *, geo_location: typing.Optional["GeoJson"] = None) -> None:
        self.geo_location = geo_location
        super().__init__(action="setGeoLocation")

    def __repr__(self) -> str:
        return "ChannelSetGeoLocationAction(action=%r, geo_location=%r)" % (
            self.action,
            self.geo_location,
        )


class ChannelSetRolesAction(ChannelUpdateAction):
    #: List of :class:`commercetools.types.ChannelRoleEnum`
    roles: typing.List["ChannelRoleEnum"]

    def __init__(self, *, roles: typing.List["ChannelRoleEnum"]) -> None:
        self.roles = roles
        super().__init__(action="setRoles")

    def __repr__(self) -> str:
        return "ChannelSetRolesAction(action=%r, roles=%r)" % (self.action, self.roles)



# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._extension import (
    ExtensionDraftSchema,
    ExtensionPagedQueryResponseSchema,
    ExtensionSchema,
    ExtensionUpdateSchema,
)
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._extension import (
    Extension,
    ExtensionDraft,
    ExtensionPagedQueryResponse,
    ExtensionUpdate,
    ExtensionUpdateAction,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _ExtensionQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _ExtensionUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _ExtensionDeleteSchema(traits.VersionedSchema, traits.ExpandableSchema):
    pass


class ExtensionService(abstract.AbstractService):
    """Extend the behavior of an API with your business logic"""

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> Extension:
        """Retrieves the representation of an extension by its id."""
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"extensions/{id}", params=params, schema_cls=ExtensionSchema
        )

    def get_by_key(self, key: str, *, expand: OptionalListStr = None) -> Extension:
        """Retrieves the representation of an extension by its key."""
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"extensions/key={key}", params=params, schema_cls=ExtensionSchema
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> ExtensionPagedQueryResponse:
        """Extend the behavior of an API with your business logic
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _ExtensionQuerySchema,
        )
        return self._client._get(
            endpoint="extensions",
            params=params,
            schema_cls=ExtensionPagedQueryResponseSchema,
        )

    def create(
        self, draft: ExtensionDraft, *, expand: OptionalListStr = None
    ) -> Extension:
        """Currently, a maximum of 25 extensions can be created per project.

        Extend the behavior of an API with your business logic
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="extensions",
            params=params,
            data_object=draft,
            request_schema_cls=ExtensionDraftSchema,
            response_schema_cls=ExtensionSchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[ExtensionUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> Extension:
        params = self._serialize_params({"expand": expand}, _ExtensionUpdateSchema)
        update_action = ExtensionUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"extensions/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=ExtensionUpdateSchema,
            response_schema_cls=ExtensionSchema,
            force_update=force_update,
        )

    def update_by_key(
        self,
        key: str,
        version: int,
        actions: typing.List[ExtensionUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> Extension:
        params = self._serialize_params({"expand": expand}, _ExtensionUpdateSchema)
        update_action = ExtensionUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"extensions/key={key}",
            params=params,
            data_object=update_action,
            request_schema_cls=ExtensionUpdateSchema,
            response_schema_cls=ExtensionSchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> Extension:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ExtensionDeleteSchema
        )
        return self._client._delete(
            endpoint=f"extensions/{id}",
            params=params,
            response_schema_cls=ExtensionSchema,
            force_delete=force_delete,
        )

    def delete_by_key(
        self,
        key: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> Extension:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ExtensionDeleteSchema
        )
        return self._client._delete(
            endpoint=f"extensions/key={key}",
            params=params,
            response_schema_cls=ExtensionSchema,
            force_delete=force_delete,
        )



from commercetools import types
from commercetools._schemas._extension import ExtensionInputSchema
from commercetools._schemas._project import ProjectSchema


def test_extension_input_schema():
    data = {
        "action": "Create",
        "resource": {
            "typeId": "order",
            "id": "dea60382-cca5-4e29-93d7-bf2bb6615501",
            "obj": {
                "type": "Order",
                "id": "dea60382-cca5-4e29-93d7-bf2bb6615501",
                "version": 1,
                "lastMessageSequenceNumber": 1,
                "createdAt": "2018-11-03T07:35:07.622Z",
                "lastModifiedAt": "2018-11-03T07:35:07.622Z",
                "customerId": "e6938b64-0721-42dc-9cc5-82ddbd43dbf2",
                "customerEmail": "m.vantellingen@labdigital.nl",
                "totalPrice": {
                    "type": "centPrecision",
                    "currencyCode": "EUR",
                    "centAmount": 10500,
                    "fractionDigits": 2,
                },
                "taxedPrice": {
                    "totalNet": {
                        "type": "centPrecision",
                        "currencyCode": "EUR",
                        "centAmount": 8677,
                        "fractionDigits": 2,
                    },
                    "totalGross": {
                        "type": "centPrecision",
                        "currencyCode": "EUR",
                        "centAmount": 10500,
                        "fractionDigits": 2,
                    },
                    "taxPortions": [
                        {
                            "rate": 0.21,
                            "amount": {
                                "type": "centPrecision",
                                "currencyCode": "EUR",
                                "centAmount": 1823,
                                "fractionDigits": 2,
                            },
                            "name": "TEMP",
                        }
                    ],
                },
                "orderState": "Open",
                "syncInfo": [],
                "returnInfo": [],
                "shippingInfo": {
                    "shippingMethodName": "TEMP",
                    "price": {
                        "type": "centPrecision",
                        "currencyCode": "EUR",
                        "centAmount": 500,
                        "fractionDigits": 2,
                    },
                    "shippingRate": {
                        "price": {
                            "type": "centPrecision",
                            "currencyCode": "EUR",
                            "centAmount": 500,
                            "fractionDigits": 2,
                        },
                        "tiers": [],
                    },
                    "taxRate": {
                        "name": "TEMP",
                        "amount": 0.21,
                        "includedInPrice": True,
                        "country": "NL",
                        "id": "viljsTSo",
                        "subRates": [],
                    },
                    "taxCategory": {
                        "typeId": "tax-category",
                        "id": "e589180a-8096-4ef5-afd1-550a3ead1d1d",
                    },
                    "deliveries": [],
                    "shippingMethod": {
                        "typeId": "shipping-method",
                        "id": "09b99a41-17f6-4e8d-9812-8790a4b6acd3",
                    },
                    "taxedPrice": {
                        "totalNet": {
                            "type": "centPrecision",
                            "currencyCode": "EUR",
                            "centAmount": 413,
                            "fractionDigits": 2,
                        },
                        "totalGross": {
                            "type": "centPrecision",
                            "currencyCode": "EUR",
                            "centAmount": 500,
                            "fractionDigits": 2,
                        },
                    },
                    "shippingMethodState": "MatchesCart",
                },
                "taxMode": "Platform",
                "inventoryMode": "None",
                "taxRoundingMode": "HalfEven",
                "taxCalculationMode": "LineItemLevel",
                "origin": "Merchant",
                "lineItems": [],
                "customLineItems": [
                    {
                        "totalPrice": {
                            "type": "centPrecision",
                            "currencyCode": "EUR",
                            "centAmount": 10000,
                            "fractionDigits": 2,
                        },
                        "id": "0cefd1ca-82b1-4dd5-ac14-20f06fb07d4d",
                        "name": {"en-GB": "Test"},
                        "money": {
                            "type": "centPrecision",
                            "currencyCode": "EUR",
                            "centAmount": 10000,
                            "fractionDigits": 2,
                        },
                        "slug": "toyota-c-hr-18-hybrid-122pk-cvt-energy",
                        "quantity": 1,
                        "discountedPricePerQuantity": [],
                        "taxCategory": {
                            "typeId": "tax-category",
                            "id": "e589180a-8096-4ef5-afd1-550a3ead1d1d",
                        },
                        "taxRate": {
                            "name": "TEMP",
                            "amount": 0.21,
                            "includedInPrice": True,
                            "country": "NL",
                            "id": "viljsTSo",
                            "subRates": [],
                        },
                        "state": [
                            {
                                "quantity": 1,
                                "state": {
                                    "typeId": "state",
                                    "id": "d09c7259-7ba4-479f-b8d2-8d2c6d2a807a",
                                },
                            }
                        ],
                        "taxedPrice": {
                            "totalNet": {
                                "type": "centPrecision",
                                "currencyCode": "EUR",
                                "centAmount": 8264,
                                "fractionDigits": 2,
                            },
                            "totalGross": {
                                "type": "centPrecision",
                                "currencyCode": "EUR",
                                "centAmount": 10000,
                                "fractionDigits": 2,
                            },
                        },
                    }
                ],
                "transactionFee": True,
                "discountCodes": [],
                "cart": {
                    "typeId": "cart",
                    "id": "737c2868-55bd-4c44-a332-f3b93122bfd7",
                },
                "shippingAddress": {
                    "id": "2x5-gO75",
                    "streetName": "Kanaalweg",
                    "streetNumber": "14",
                    "postalCode": "3526KL",
                    "city": "Utrecht",
                    "country": "NL",
                },
                "billingAddress": {
                    "id": "2x5-gO75",
                    "streetName": "Kanaalweg",
                    "streetNumber": "14",
                    "postalCode": "3526KL",
                    "city": "Utrecht",
                    "country": "NL",
                },
                "itemShippingAddresses": [],
                "refusedGifts": [],
            },
        },
    }
    result = ExtensionInputSchema().load(data)
    assert result.action == types.ExtensionAction.CREATE
    assert result.resource.type_id == types.ReferenceTypeId.ORDER

    data = ExtensionInputSchema().dump(result)


def test_project_schema():
    data = {
        "key": "unittest",
        "name": "SDK Test",
        "countries": ["AT", "NL", "CH", "BE", "GB", "DE", "IT", "LU", "ES"],
        "currencies": ["EUR", "GBP", "CHF"],
        "languages": [
            "en-GB",
            "de-DE",
            "fr-FR",
            "nl-BE",
            "nl-NL",
            "fr-BE",
            "it-IT",
            "es-ES",
            "pt-PT",
        ],
        "createdAt": "2018-10-24T08:58:22.935Z",
        "carts": {"countryTaxRateFallbackEnabled": True},
        "trialUntil": "2018-12",
        "messages": {"enabled": False, "deleteDaysAfterCreation": 15},
        "version": 9,
    }

    result = ProjectSchema().load(data)
    data = ProjectSchema().dump(result)



# DO NOT EDIT! This file is automatically generated
import datetime
import enum
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import BaseResource

if typing.TYPE_CHECKING:
    from ._common import CreatedBy, LastModifiedBy, Reference
    from ._message import UserProvidedIdentifiers
__all__ = [
    "AzureEventGridDestination",
    "AzureServiceBusDestination",
    "ChangeSubscription",
    "DeliveryCloudEventsFormat",
    "DeliveryFormat",
    "DeliveryPlatformFormat",
    "Destination",
    "GoogleCloudPubSubDestination",
    "IronMqDestination",
    "MessageDelivery",
    "MessageSubscription",
    "PayloadNotIncluded",
    "ResourceCreatedDelivery",
    "ResourceDeletedDelivery",
    "ResourceUpdatedDelivery",
    "SnsDestination",
    "SqsDestination",
    "Subscription",
    "SubscriptionChangeDestinationAction",
    "SubscriptionDelivery",
    "SubscriptionDraft",
    "SubscriptionHealthStatus",
    "SubscriptionPagedQueryResponse",
    "SubscriptionSetChangesAction",
    "SubscriptionSetKeyAction",
    "SubscriptionSetMessagesAction",
    "SubscriptionUpdate",
    "SubscriptionUpdateAction",
]


class ChangeSubscription(_BaseType):
    #: :class:`str` `(Named` ``resourceTypeId`` `in Commercetools)`
    resource_type_id: str

    def __init__(self, *, resource_type_id: str) -> None:
        self.resource_type_id = resource_type_id
        super().__init__()

    def __repr__(self) -> str:
        return "ChangeSubscription(resource_type_id=%r)" % (self.resource_type_id,)


class DeliveryFormat(_BaseType):
    #: :class:`str`
    type: str

    def __init__(self, *, type: str) -> None:
        self.type = type
        super().__init__()

    def __repr__(self) -> str:
        return "DeliveryFormat(type=%r)" % (self.type,)


class Destination(_BaseType):
    #: :class:`str`
    type: str

    def __init__(self, *, type: str) -> None:
        self.type = type
        super().__init__()

    def __repr__(self) -> str:
        return "Destination(type=%r)" % (self.type,)


class MessageSubscription(_BaseType):
    #: :class:`str` `(Named` ``resourceTypeId`` `in Commercetools)`
    resource_type_id: str
    #: Optional list of :class:`str`
    types: typing.Optional[typing.List[str]]

    def __init__(
        self, *, resource_type_id: str, types: typing.Optional[typing.List[str]] = None
    ) -> None:
        self.resource_type_id = resource_type_id
        self.types = types
        super().__init__()

    def __repr__(self) -> str:
        return "MessageSubscription(resource_type_id=%r, types=%r)" % (
            self.resource_type_id,
            self.types,
        )


class PayloadNotIncluded(_BaseType):
    #: :class:`str`
    reason: str
    #: :class:`str` `(Named` ``payloadType`` `in Commercetools)`
    payload_type: str

    def __init__(self, *, reason: str, payload_type: str) -> None:
        self.reason = reason
        self.payload_type = payload_type
        super().__init__()

    def __repr__(self) -> str:
        return "PayloadNotIncluded(reason=%r, payload_type=%r)" % (
            self.reason,
            self.payload_type,
        )


class Subscription(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: List of :class:`commercetools.types.ChangeSubscription`
    changes: typing.List["ChangeSubscription"]
    #: :class:`commercetools.types.Destination`
    destination: "Destination"
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: List of :class:`commercetools.types.MessageSubscription`
    messages: typing.List["MessageSubscription"]
    #: :class:`commercetools.types.DeliveryFormat`
    format: "DeliveryFormat"
    #: :class:`commercetools.types.SubscriptionHealthStatus`
    status: "SubscriptionHealthStatus"

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        changes: typing.List["ChangeSubscription"],
        destination: "Destination",
        messages: typing.List["MessageSubscription"],
        format: "DeliveryFormat",
        status: "SubscriptionHealthStatus",
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        key: typing.Optional[str] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.changes = changes
        self.destination = destination
        self.key = key
        self.messages = messages
        self.format = format
        self.status = status
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "Subscription(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, changes=%r, destination=%r, key=%r, messages=%r, format=%r, status=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.changes,
                self.destination,
                self.key,
                self.messages,
                self.format,
                self.status,
            )
        )


class SubscriptionDelivery(_BaseType):
    #: :class:`str` `(Named` ``projectKey`` `in Commercetools)`
    project_key: str
    #: :class:`str` `(Named` ``notificationType`` `in Commercetools)`
    notification_type: str
    #: :class:`commercetools.types.Reference`
    resource: "Reference"
    #: Optional :class:`commercetools.types.UserProvidedIdentifiers` `(Named` ``resourceUserProvidedIdentifiers`` `in Commercetools)`
    resource_user_provided_identifiers: typing.Optional["UserProvidedIdentifiers"]

    def __init__(
        self,
        *,
        project_key: str,
        notification_type: str,
        resource: "Reference",
        resource_user_provided_identifiers: typing.Optional[
            "UserProvidedIdentifiers"
        ] = None
    ) -> None:
        self.project_key = project_key
        self.notification_type = notification_type
        self.resource = resource
        self.resource_user_provided_identifiers = resource_user_provided_identifiers
        super().__init__()

    def __repr__(self) -> str:
        return (
            "SubscriptionDelivery(project_key=%r, notification_type=%r, resource=%r, resource_user_provided_identifiers=%r)"
            % (
                self.project_key,
                self.notification_type,
                self.resource,
                self.resource_user_provided_identifiers,
            )
        )


class SubscriptionDraft(_BaseType):
    #: Optional list of :class:`commercetools.types.ChangeSubscription`
    changes: typing.Optional[typing.List["ChangeSubscription"]]
    #: :class:`commercetools.types.Destination`
    destination: "Destination"
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.MessageSubscription`
    messages: typing.Optional[typing.List["MessageSubscription"]]
    #: Optional :class:`commercetools.types.DeliveryFormat`
    format: typing.Optional["DeliveryFormat"]

    def __init__(
        self,
        *,
        destination: "Destination",
        changes: typing.Optional[typing.List["ChangeSubscription"]] = None,
        key: typing.Optional[str] = None,
        messages: typing.Optional[typing.List["MessageSubscription"]] = None,
        format: typing.Optional["DeliveryFormat"] = None
    ) -> None:
        self.changes = changes
        self.destination = destination
        self.key = key
        self.messages = messages
        self.format = format
        super().__init__()

    def __repr__(self) -> str:
        return (
            "SubscriptionDraft(changes=%r, destination=%r, key=%r, messages=%r, format=%r)"
            % (self.changes, self.destination, self.key, self.messages, self.format)
        )


class SubscriptionHealthStatus(enum.Enum):
    HEALTHY = "Healthy"
    CONFIGURATION_ERROR = "ConfigurationError"
    CONFIGURATION_ERROR_DELIVERY_STOPPED = "ConfigurationErrorDeliveryStopped"
    TEMPORARY_ERROR = "TemporaryError"


class SubscriptionPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.Subscription`
    results: typing.Sequence["Subscription"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["Subscription"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "SubscriptionPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class SubscriptionUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "SubscriptionUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class SubscriptionUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "SubscriptionUpdateAction(action=%r)" % (self.action,)


class AzureEventGridDestination(Destination):
    #: :class:`str`
    uri: str
    #: :class:`str` `(Named` ``accessKey`` `in Commercetools)`
    access_key: str

    def __init__(self, *, uri: str, access_key: str) -> None:
        self.uri = uri
        self.access_key = access_key
        super().__init__(type="EventGrid")

    def __repr__(self) -> str:
        return "AzureEventGridDestination(type=%r, uri=%r, access_key=%r)" % (
            self.type,
            self.uri,
            self.access_key,
        )


class AzureServiceBusDestination(Destination):
    #: :class:`str` `(Named` ``connectionString`` `in Commercetools)`
    connection_string: str

    def __init__(self, *, connection_string: str) -> None:
        self.connection_string = connection_string
        super().__init__(type="AzureServiceBus")

    def __repr__(self) -> str:
        return "AzureServiceBusDestination(type=%r, connection_string=%r)" % (
            self.type,
            self.connection_string,
        )


class DeliveryCloudEventsFormat(DeliveryFormat):
    #: :class:`str` `(Named` ``cloudEventsVersion`` `in Commercetools)`
    cloud_events_version: str

    def __init__(self, *, cloud_events_version: str) -> None:
        self.cloud_events_version = cloud_events_version
        super().__init__(type="CloudEvents")

    def __repr__(self) -> str:
        return "DeliveryCloudEventsFormat(type=%r, cloud_events_version=%r)" % (
            self.type,
            self.cloud_events_version,
        )


class DeliveryPlatformFormat(DeliveryFormat):
    def __init__(self) -> None:
        super().__init__(type="Platform")

    def __repr__(self) -> str:
        return "DeliveryPlatformFormat(type=%r)" % (self.type,)


class GoogleCloudPubSubDestination(Destination):
    #: :class:`str` `(Named` ``projectId`` `in Commercetools)`
    project_id: str
    #: :class:`str`
    topic: str

    def __init__(self, *, project_id: str, topic: str) -> None:
        self.project_id = project_id
        self.topic = topic
        super().__init__(type="GoogleCloudPubSub")

    def __repr__(self) -> str:
        return "GoogleCloudPubSubDestination(type=%r, project_id=%r, topic=%r)" % (
            self.type,
            self.project_id,
            self.topic,
        )


class IronMqDestination(Destination):
    #: :class:`str`
    uri: str

    def __init__(self, *, uri: str) -> None:
        self.uri = uri
        super().__init__(type="IronMQ")

    def __repr__(self) -> str:
        return "IronMqDestination(type=%r, uri=%r)" % (self.type, self.uri)


class MessageDelivery(SubscriptionDelivery):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: :class:`int` `(Named` ``sequenceNumber`` `in Commercetools)`
    sequence_number: int
    #: :class:`int` `(Named` ``resourceVersion`` `in Commercetools)`
    resource_version: int
    #: :class:`commercetools.types.PayloadNotIncluded` `(Named` ``payloadNotIncluded`` `in Commercetools)`
    payload_not_included: "PayloadNotIncluded"

    def __init__(
        self,
        *,
        project_key: str,
        resource: "Reference",
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        sequence_number: int,
        resource_version: int,
        payload_not_included: "PayloadNotIncluded",
        resource_user_provided_identifiers: typing.Optional[
            "UserProvidedIdentifiers"
        ] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.sequence_number = sequence_number
        self.resource_version = resource_version
        self.payload_not_included = payload_not_included
        super().__init__(
            project_key=project_key,
            notification_type="Message",
            resource=resource,
            resource_user_provided_identifiers=resource_user_provided_identifiers,
        )

    def __repr__(self) -> str:
        return (
            "MessageDelivery(project_key=%r, notification_type=%r, resource=%r, resource_user_provided_identifiers=%r, id=%r, version=%r, created_at=%r, last_modified_at=%r, sequence_number=%r, resource_version=%r, payload_not_included=%r)"
            % (
                self.project_key,
                self.notification_type,
                self.resource,
                self.resource_user_provided_identifiers,
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.sequence_number,
                self.resource_version,
                self.payload_not_included,
            )
        )


class ResourceCreatedDelivery(SubscriptionDelivery):
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``modifiedAt`` `in Commercetools)`
    modified_at: datetime.datetime

    def __init__(
        self,
        *,
        project_key: str,
        resource: "Reference",
        version: int,
        modified_at: datetime.datetime,
        resource_user_provided_identifiers: typing.Optional[
            "UserProvidedIdentifiers"
        ] = None
    ) -> None:
        self.version = version
        self.modified_at = modified_at
        super().__init__(
            project_key=project_key,
            notification_type="ResourceCreated",
            resource=resource,
            resource_user_provided_identifiers=resource_user_provided_identifiers,
        )

    def __repr__(self) -> str:
        return (
            "ResourceCreatedDelivery(project_key=%r, notification_type=%r, resource=%r, resource_user_provided_identifiers=%r, version=%r, modified_at=%r)"
            % (
                self.project_key,
                self.notification_type,
                self.resource,
                self.resource_user_provided_identifiers,
                self.version,
                self.modified_at,
            )
        )


class ResourceDeletedDelivery(SubscriptionDelivery):
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``modifiedAt`` `in Commercetools)`
    modified_at: datetime.datetime

    def __init__(
        self,
        *,
        project_key: str,
        resource: "Reference",
        version: int,
        modified_at: datetime.datetime,
        resource_user_provided_identifiers: typing.Optional[
            "UserProvidedIdentifiers"
        ] = None
    ) -> None:
        self.version = version
        self.modified_at = modified_at
        super().__init__(
            project_key=project_key,
            notification_type="ResourceDeleted",
            resource=resource,
            resource_user_provided_identifiers=resource_user_provided_identifiers,
        )

    def __repr__(self) -> str:
        return (
            "ResourceDeletedDelivery(project_key=%r, notification_type=%r, resource=%r, resource_user_provided_identifiers=%r, version=%r, modified_at=%r)"
            % (
                self.project_key,
                self.notification_type,
                self.resource,
                self.resource_user_provided_identifiers,
                self.version,
                self.modified_at,
            )
        )


class ResourceUpdatedDelivery(SubscriptionDelivery):
    #: :class:`int`
    version: int
    #: :class:`int` `(Named` ``oldVersion`` `in Commercetools)`
    old_version: int
    #: :class:`datetime.datetime` `(Named` ``modifiedAt`` `in Commercetools)`
    modified_at: datetime.datetime

    def __init__(
        self,
        *,
        project_key: str,
        resource: "Reference",
        version: int,
        old_version: int,
        modified_at: datetime.datetime,
        resource_user_provided_identifiers: typing.Optional[
            "UserProvidedIdentifiers"
        ] = None
    ) -> None:
        self.version = version
        self.old_version = old_version
        self.modified_at = modified_at
        super().__init__(
            project_key=project_key,
            notification_type="ResourceUpdated",
            resource=resource,
            resource_user_provided_identifiers=resource_user_provided_identifiers,
        )

    def __repr__(self) -> str:
        return (
            "ResourceUpdatedDelivery(project_key=%r, notification_type=%r, resource=%r, resource_user_provided_identifiers=%r, version=%r, old_version=%r, modified_at=%r)"
            % (
                self.project_key,
                self.notification_type,
                self.resource,
                self.resource_user_provided_identifiers,
                self.version,
                self.old_version,
                self.modified_at,
            )
        )


class SnsDestination(Destination):
    #: :class:`str` `(Named` ``accessKey`` `in Commercetools)`
    access_key: str
    #: :class:`str` `(Named` ``accessSecret`` `in Commercetools)`
    access_secret: str
    #: :class:`str` `(Named` ``topicArn`` `in Commercetools)`
    topic_arn: str

    def __init__(self, *, access_key: str, access_secret: str, topic_arn: str) -> None:
        self.access_key = access_key
        self.access_secret = access_secret
        self.topic_arn = topic_arn
        super().__init__(type="SNS")

    def __repr__(self) -> str:
        return (
            "SnsDestination(type=%r, access_key=%r, access_secret=%r, topic_arn=%r)"
            % (self.type, self.access_key, self.access_secret, self.topic_arn)
        )


class SqsDestination(Destination):
    #: :class:`str` `(Named` ``accessKey`` `in Commercetools)`
    access_key: str
    #: :class:`str` `(Named` ``accessSecret`` `in Commercetools)`
    access_secret: str
    #: :class:`str` `(Named` ``queueUrl`` `in Commercetools)`
    queue_url: str
    #: :class:`str`
    region: str

    def __init__(
        self, *, access_key: str, access_secret: str, queue_url: str, region: str
    ) -> None:
        self.access_key = access_key
        self.access_secret = access_secret
        self.queue_url = queue_url
        self.region = region
        super().__init__(type="SQS")

    def __repr__(self) -> str:
        return (
            "SqsDestination(type=%r, access_key=%r, access_secret=%r, queue_url=%r, region=%r)"
            % (
                self.type,
                self.access_key,
                self.access_secret,
                self.queue_url,
                self.region,
            )
        )


class SubscriptionChangeDestinationAction(SubscriptionUpdateAction):
    #: :class:`commercetools.types.Destination`
    destination: "Destination"

    def __init__(self, *, destination: "Destination") -> None:
        self.destination = destination
        super().__init__(action="changeDestination")

    def __repr__(self) -> str:
        return "SubscriptionChangeDestinationAction(action=%r, destination=%r)" % (
            self.action,
            self.destination,
        )


class SubscriptionSetChangesAction(SubscriptionUpdateAction):
    #: Optional list of :class:`commercetools.types.ChangeSubscription`
    changes: typing.Optional[typing.List["ChangeSubscription"]]

    def __init__(
        self, *, changes: typing.Optional[typing.List["ChangeSubscription"]] = None
    ) -> None:
        self.changes = changes
        super().__init__(action="setChanges")

    def __repr__(self) -> str:
        return "SubscriptionSetChangesAction(action=%r, changes=%r)" % (
            self.action,
            self.changes,
        )


class SubscriptionSetKeyAction(SubscriptionUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "SubscriptionSetKeyAction(action=%r, key=%r)" % (self.action, self.key)


class SubscriptionSetMessagesAction(SubscriptionUpdateAction):
    #: Optional list of :class:`commercetools.types.MessageSubscription`
    messages: typing.Optional[typing.List["MessageSubscription"]]

    def __init__(
        self, *, messages: typing.Optional[typing.List["MessageSubscription"]] = None
    ) -> None:
        self.messages = messages
        super().__init__(action="setMessages")

    def __repr__(self) -> str:
        return "SubscriptionSetMessagesAction(action=%r, messages=%r)" % (
            self.action,
            self.messages,
        )



import pytest

# Tricks flake8 into silencing redefining fixtures warnings.
pytest_plugins = ["commercetools.contrib.pytest"]


@pytest.fixture
def client(commercetools_client):
    return commercetools_client



# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "CartAddCustomLineItemActionSchema",
    "CartAddDiscountCodeActionSchema",
    "CartAddItemShippingAddressActionSchema",
    "CartAddLineItemActionSchema",
    "CartAddPaymentActionSchema",
    "CartAddShoppingListActionSchema",
    "CartApplyDeltaToCustomLineItemShippingDetailsTargetsActionSchema",
    "CartApplyDeltaToLineItemShippingDetailsTargetsActionSchema",
    "CartChangeCustomLineItemMoneyActionSchema",
    "CartChangeCustomLineItemQuantityActionSchema",
    "CartChangeLineItemQuantityActionSchema",
    "CartChangeTaxCalculationModeActionSchema",
    "CartChangeTaxModeActionSchema",
    "CartChangeTaxRoundingModeActionSchema",
    "CartDraftSchema",
    "CartPagedQueryResponseSchema",
    "CartRecalculateActionSchema",
    "CartReferenceSchema",
    "CartRemoveCustomLineItemActionSchema",
    "CartRemoveDiscountCodeActionSchema",
    "CartRemoveItemShippingAddressActionSchema",
    "CartRemoveLineItemActionSchema",
    "CartRemovePaymentActionSchema",
    "CartResourceIdentifierSchema",
    "CartSchema",
    "CartSetAnonymousIdActionSchema",
    "CartSetBillingAddressActionSchema",
    "CartSetCartTotalTaxActionSchema",
    "CartSetCountryActionSchema",
    "CartSetCustomFieldActionSchema",
    "CartSetCustomLineItemCustomFieldActionSchema",
    "CartSetCustomLineItemCustomTypeActionSchema",
    "CartSetCustomLineItemShippingDetailsActionSchema",
    "CartSetCustomLineItemTaxAmountActionSchema",
    "CartSetCustomLineItemTaxRateActionSchema",
    "CartSetCustomShippingMethodActionSchema",
    "CartSetCustomTypeActionSchema",
    "CartSetCustomerEmailActionSchema",
    "CartSetCustomerGroupActionSchema",
    "CartSetCustomerIdActionSchema",
    "CartSetDeleteDaysAfterLastModificationActionSchema",
    "CartSetLineItemCustomFieldActionSchema",
    "CartSetLineItemCustomTypeActionSchema",
    "CartSetLineItemDistributionChannelActionSchema",
    "CartSetLineItemPriceActionSchema",
    "CartSetLineItemShippingDetailsActionSchema",
    "CartSetLineItemTaxAmountActionSchema",
    "CartSetLineItemTaxRateActionSchema",
    "CartSetLineItemTotalPriceActionSchema",
    "CartSetLocaleActionSchema",
    "CartSetShippingAddressActionSchema",
    "CartSetShippingMethodActionSchema",
    "CartSetShippingMethodTaxAmountActionSchema",
    "CartSetShippingMethodTaxRateActionSchema",
    "CartSetShippingRateInputActionSchema",
    "CartUpdateActionSchema",
    "CartUpdateItemShippingAddressActionSchema",
    "CartUpdateSchema",
    "ClassificationShippingRateInputDraftSchema",
    "ClassificationShippingRateInputSchema",
    "CustomLineItemDraftSchema",
    "CustomLineItemSchema",
    "DiscountCodeInfoSchema",
    "DiscountedLineItemPortionSchema",
    "DiscountedLineItemPriceForQuantitySchema",
    "DiscountedLineItemPriceSchema",
    "ExternalLineItemTotalPriceSchema",
    "ExternalTaxAmountDraftSchema",
    "ExternalTaxRateDraftSchema",
    "ItemShippingDetailsDraftSchema",
    "ItemShippingDetailsSchema",
    "ItemShippingTargetSchema",
    "LineItemDraftSchema",
    "LineItemSchema",
    "ReplicaCartDraftSchema",
    "ScoreShippingRateInputDraftSchema",
    "ScoreShippingRateInputSchema",
    "ShippingInfoSchema",
    "ShippingRateInputDraftSchema",
    "ShippingRateInputSchema",
    "TaxPortionDraftSchema",
    "TaxPortionSchema",
    "TaxedItemPriceSchema",
    "TaxedPriceDraftSchema",
    "TaxedPriceSchema",
]


class CartDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDraft`."""

    currency = marshmallow.fields.String()
    customer_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerId"
    )
    customer_email = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerEmail"
    )
    customer_group = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="customerGroup",
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )
    store = helpers.LazyNestedField(
        nested="commercetools._schemas._store.StoreResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    country = marshmallow.fields.String(allow_none=True, missing=None)
    inventory_mode = marshmallow_enum.EnumField(
        types.InventoryMode, by_value=True, missing=None, data_key="inventoryMode"
    )
    tax_mode = marshmallow_enum.EnumField(
        types.TaxMode, by_value=True, missing=None, data_key="taxMode"
    )
    tax_rounding_mode = marshmallow_enum.EnumField(
        types.RoundingMode, by_value=True, missing=None, data_key="taxRoundingMode"
    )
    tax_calculation_mode = marshmallow_enum.EnumField(
        types.TaxCalculationMode,
        by_value=True,
        missing=None,
        data_key="taxCalculationMode",
    )
    line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.LineItemDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="lineItems",
    )
    custom_line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.CustomLineItemDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="customLineItems",
    )
    shipping_address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingAddress",
    )
    billing_address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="billingAddress",
    )
    shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingMethod",
    )
    external_tax_rate_for_shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRateForShippingMethod",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    locale = marshmallow.fields.String(allow_none=True, missing=None)
    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )
    origin = marshmallow_enum.EnumField(types.CartOrigin, by_value=True, missing=None)
    shipping_rate_input = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "Classification": "commercetools._schemas._cart.ClassificationShippingRateInputDraftSchema",
            "Score": "commercetools._schemas._cart.ScoreShippingRateInputDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingRateInput",
    )
    item_shipping_addresses = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="itemShippingAddresses",
    )
    discount_codes = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True),
        missing=None,
        data_key="discountCodes",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartDraft(**data)


class CartPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.CartSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartPagedQueryResponse(**data)


class CartReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.CartReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.CartSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CartReference(**data)


class CartResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.CartResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CartResourceIdentifier(**data)


class CartSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Cart`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    customer_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerId"
    )
    customer_email = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerEmail"
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )
    store = helpers.LazyNestedField(
        nested="commercetools._schemas._store.StoreKeyReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.LineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="lineItems",
    )
    custom_line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.CustomLineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="customLineItems",
    )
    total_price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalPrice",
    )
    taxed_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxedPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxedPrice",
    )
    cart_state = marshmallow_enum.EnumField(
        types.CartState, by_value=True, data_key="cartState"
    )
    shipping_address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingAddress",
    )
    billing_address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="billingAddress",
    )
    inventory_mode = marshmallow_enum.EnumField(
        types.InventoryMode, by_value=True, missing=None, data_key="inventoryMode"
    )
    tax_mode = marshmallow_enum.EnumField(
        types.TaxMode, by_value=True, data_key="taxMode"
    )
    tax_rounding_mode = marshmallow_enum.EnumField(
        types.RoundingMode, by_value=True, data_key="taxRoundingMode"
    )
    tax_calculation_mode = marshmallow_enum.EnumField(
        types.TaxCalculationMode, by_value=True, data_key="taxCalculationMode"
    )
    customer_group = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="customerGroup",
    )
    country = marshmallow.fields.String(missing=None)
    shipping_info = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ShippingInfoSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingInfo",
    )
    discount_codes = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountCodeInfoSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="discountCodes",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    payment_info = helpers.LazyNestedField(
        nested="commercetools._schemas._order.PaymentInfoSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="paymentInfo",
    )
    locale = marshmallow.fields.String(allow_none=True, missing=None)
    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )
    refused_gifts = helpers.LazyNestedField(
        nested="commercetools._schemas._cart_discount.CartDiscountReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="refusedGifts",
    )
    origin = marshmallow_enum.EnumField(types.CartOrigin, by_value=True)
    shipping_rate_input = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "Classification": "commercetools._schemas._cart.ClassificationShippingRateInputSchema",
            "Score": "commercetools._schemas._cart.ScoreShippingRateInputSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingRateInput",
    )
    item_shipping_addresses = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="itemShippingAddresses",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Cart(**data)


class CartUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartUpdateAction(**data)


class CartUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addCustomLineItem": "commercetools._schemas._cart.CartAddCustomLineItemActionSchema",
                "addDiscountCode": "commercetools._schemas._cart.CartAddDiscountCodeActionSchema",
                "addItemShippingAddress": "commercetools._schemas._cart.CartAddItemShippingAddressActionSchema",
                "addLineItem": "commercetools._schemas._cart.CartAddLineItemActionSchema",
                "addPayment": "commercetools._schemas._cart.CartAddPaymentActionSchema",
                "addShoppingList": "commercetools._schemas._cart.CartAddShoppingListActionSchema",
                "applyDeltaToCustomLineItemShippingDetailsTargets": "commercetools._schemas._cart.CartApplyDeltaToCustomLineItemShippingDetailsTargetsActionSchema",
                "applyDeltaToLineItemShippingDetailsTargets": "commercetools._schemas._cart.CartApplyDeltaToLineItemShippingDetailsTargetsActionSchema",
                "changeCustomLineItemMoney": "commercetools._schemas._cart.CartChangeCustomLineItemMoneyActionSchema",
                "changeCustomLineItemQuantity": "commercetools._schemas._cart.CartChangeCustomLineItemQuantityActionSchema",
                "changeLineItemQuantity": "commercetools._schemas._cart.CartChangeLineItemQuantityActionSchema",
                "changeTaxCalculationMode": "commercetools._schemas._cart.CartChangeTaxCalculationModeActionSchema",
                "changeTaxMode": "commercetools._schemas._cart.CartChangeTaxModeActionSchema",
                "changeTaxRoundingMode": "commercetools._schemas._cart.CartChangeTaxRoundingModeActionSchema",
                "recalculate": "commercetools._schemas._cart.CartRecalculateActionSchema",
                "removeCustomLineItem": "commercetools._schemas._cart.CartRemoveCustomLineItemActionSchema",
                "removeDiscountCode": "commercetools._schemas._cart.CartRemoveDiscountCodeActionSchema",
                "removeItemShippingAddress": "commercetools._schemas._cart.CartRemoveItemShippingAddressActionSchema",
                "removeLineItem": "commercetools._schemas._cart.CartRemoveLineItemActionSchema",
                "removePayment": "commercetools._schemas._cart.CartRemovePaymentActionSchema",
                "setAnonymousId": "commercetools._schemas._cart.CartSetAnonymousIdActionSchema",
                "setBillingAddress": "commercetools._schemas._cart.CartSetBillingAddressActionSchema",
                "setCartTotalTax": "commercetools._schemas._cart.CartSetCartTotalTaxActionSchema",
                "setCountry": "commercetools._schemas._cart.CartSetCountryActionSchema",
                "setCustomField": "commercetools._schemas._cart.CartSetCustomFieldActionSchema",
                "setCustomLineItemCustomField": "commercetools._schemas._cart.CartSetCustomLineItemCustomFieldActionSchema",
                "setCustomLineItemCustomType": "commercetools._schemas._cart.CartSetCustomLineItemCustomTypeActionSchema",
                "setCustomLineItemShippingDetails": "commercetools._schemas._cart.CartSetCustomLineItemShippingDetailsActionSchema",
                "setCustomLineItemTaxAmount": "commercetools._schemas._cart.CartSetCustomLineItemTaxAmountActionSchema",
                "setCustomLineItemTaxRate": "commercetools._schemas._cart.CartSetCustomLineItemTaxRateActionSchema",
                "setCustomShippingMethod": "commercetools._schemas._cart.CartSetCustomShippingMethodActionSchema",
                "setCustomType": "commercetools._schemas._cart.CartSetCustomTypeActionSchema",
                "setCustomerEmail": "commercetools._schemas._cart.CartSetCustomerEmailActionSchema",
                "setCustomerGroup": "commercetools._schemas._cart.CartSetCustomerGroupActionSchema",
                "setCustomerId": "commercetools._schemas._cart.CartSetCustomerIdActionSchema",
                "setDeleteDaysAfterLastModification": "commercetools._schemas._cart.CartSetDeleteDaysAfterLastModificationActionSchema",
                "setLineItemCustomField": "commercetools._schemas._cart.CartSetLineItemCustomFieldActionSchema",
                "setLineItemCustomType": "commercetools._schemas._cart.CartSetLineItemCustomTypeActionSchema",
                "setLineItemDistributionChannel": "commercetools._schemas._cart.CartSetLineItemDistributionChannelActionSchema",
                "setLineItemPrice": "commercetools._schemas._cart.CartSetLineItemPriceActionSchema",
                "setLineItemShippingDetails": "commercetools._schemas._cart.CartSetLineItemShippingDetailsActionSchema",
                "setLineItemTaxAmount": "commercetools._schemas._cart.CartSetLineItemTaxAmountActionSchema",
                "setLineItemTaxRate": "commercetools._schemas._cart.CartSetLineItemTaxRateActionSchema",
                "setLineItemTotalPrice": "commercetools._schemas._cart.CartSetLineItemTotalPriceActionSchema",
                "setLocale": "commercetools._schemas._cart.CartSetLocaleActionSchema",
                "setShippingAddress": "commercetools._schemas._cart.CartSetShippingAddressActionSchema",
                "setShippingMethod": "commercetools._schemas._cart.CartSetShippingMethodActionSchema",
                "setShippingMethodTaxAmount": "commercetools._schemas._cart.CartSetShippingMethodTaxAmountActionSchema",
                "setShippingMethodTaxRate": "commercetools._schemas._cart.CartSetShippingMethodTaxRateActionSchema",
                "setShippingRateInput": "commercetools._schemas._cart.CartSetShippingRateInputActionSchema",
                "updateItemShippingAddress": "commercetools._schemas._cart.CartUpdateItemShippingAddressActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartUpdate(**data)


class CustomLineItemDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomLineItemDraft`."""

    name = LocalizedStringField(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True)
    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    slug = marshmallow.fields.String(allow_none=True)
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomLineItemDraft(**data)


class CustomLineItemSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomLineItem`."""

    id = marshmallow.fields.String(allow_none=True)
    name = LocalizedStringField(allow_none=True)
    money = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    taxed_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxedItemPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxedPrice",
    )
    total_price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalPrice",
    )
    slug = marshmallow.fields.String(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True)
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ItemStateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxRate",
    )
    discounted_price_per_quantity = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountedLineItemPriceForQuantitySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="discountedPricePerQuantity",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomLineItem(**data)


class DiscountCodeInfoSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.DiscountCodeInfo`."""

    discount_code = helpers.LazyNestedField(
        nested="commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="discountCode",
    )
    state = marshmallow_enum.EnumField(types.DiscountCodeState, by_value=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.DiscountCodeInfo(**data)


class DiscountedLineItemPortionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.DiscountedLineItemPortion`."""

    discount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart_discount.CartDiscountReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    discounted_amount = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="discountedAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.DiscountedLineItemPortion(**data)


class DiscountedLineItemPriceForQuantitySchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.DiscountedLineItemPriceForQuantity`."""

    quantity = marshmallow.fields.Integer(allow_none=True)
    discounted_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountedLineItemPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="discountedPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.DiscountedLineItemPriceForQuantity(**data)


class DiscountedLineItemPriceSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.DiscountedLineItemPrice`."""

    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    included_discounts = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountedLineItemPortionSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="includedDiscounts",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.DiscountedLineItemPrice(**data)


class ExternalLineItemTotalPriceSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExternalLineItemTotalPrice`."""

    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExternalLineItemTotalPrice(**data)


class ExternalTaxAmountDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExternalTaxAmountDraft`."""

    total_gross = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalGross",
    )
    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExternalTaxAmountDraft(**data)


class ExternalTaxRateDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExternalTaxRateDraft`."""

    name = marshmallow.fields.String(allow_none=True)
    amount = marshmallow.fields.Integer(allow_none=True, missing=None)
    country = marshmallow.fields.String(allow_none=True)
    state = marshmallow.fields.String(allow_none=True, missing=None)
    sub_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.SubRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="subRates",
    )
    included_in_price = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="includedInPrice"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExternalTaxRateDraft(**data)


class ItemShippingDetailsDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ItemShippingDetailsDraft`."""

    targets = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingTargetSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ItemShippingDetailsDraft(**data)


class ItemShippingDetailsSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ItemShippingDetails`."""

    targets = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingTargetSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    valid = marshmallow.fields.Bool(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ItemShippingDetails(**data)


class ItemShippingTargetSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ItemShippingTarget`."""

    address_key = marshmallow.fields.String(allow_none=True, data_key="addressKey")
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ItemShippingTarget(**data)


class LineItemDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.LineItemDraft`."""

    product_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="productId"
    )
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )
    sku = marshmallow.fields.String(allow_none=True, missing=None)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.LineItemDraft(**data)


class LineItemSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.LineItem`."""

    id = marshmallow.fields.String(allow_none=True)
    product_id = marshmallow.fields.String(allow_none=True, data_key="productId")
    name = LocalizedStringField(allow_none=True)
    product_slug = LocalizedStringField(
        allow_none=True, missing=None, data_key="productSlug"
    )
    product_type = helpers.LazyNestedField(
        nested="commercetools._schemas._product_type.ProductTypeReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="productType",
    )
    variant = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductVariantSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.PriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    taxed_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxedItemPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxedPrice",
    )
    total_price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalPrice",
    )
    quantity = marshmallow.fields.Integer(allow_none=True)
    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ItemStateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxRate",
    )
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )
    discounted_price_per_quantity = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountedLineItemPriceForQuantitySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="discountedPricePerQuantity",
    )
    price_mode = marshmallow_enum.EnumField(
        types.LineItemPriceMode, by_value=True, data_key="priceMode"
    )
    line_item_mode = marshmallow_enum.EnumField(
        types.LineItemMode, by_value=True, data_key="lineItemMode"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.LineItem(**data)


class ReplicaCartDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReplicaCartDraft`."""

    reference = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.CartReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ReplicaCartDraft(**data)


class ShippingInfoSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingInfo`."""

    shipping_method_name = marshmallow.fields.String(
        allow_none=True, data_key="shippingMethodName"
    )
    price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )
    taxed_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxedItemPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxedPrice",
    )
    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxRate",
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingMethod",
    )
    deliveries = helpers.LazyNestedField(
        nested="commercetools._schemas._order.DeliverySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )
    discounted_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.DiscountedLineItemPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="discountedPrice",
    )
    shipping_method_state = marshmallow_enum.EnumField(
        types.ShippingMethodState, by_value=True, data_key="shippingMethodState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingInfo(**data)


class ShippingRateInputDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingRateInputDraft`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ShippingRateInputDraft(**data)


class ShippingRateInputSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingRateInput`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ShippingRateInput(**data)


class TaxPortionDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxPortionDraft`."""

    name = marshmallow.fields.String(allow_none=True, missing=None)
    rate = marshmallow.fields.Float(allow_none=True)
    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxPortionDraft(**data)


class TaxPortionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxPortion`."""

    name = marshmallow.fields.String(allow_none=True, missing=None)
    rate = marshmallow.fields.Float(allow_none=True)
    amount = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxPortion(**data)


class TaxedItemPriceSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxedItemPrice`."""

    total_net = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalNet",
    )
    total_gross = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalGross",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxedItemPrice(**data)


class TaxedPriceDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxedPriceDraft`."""

    total_net = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneyDraftSchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneyDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalNet",
    )
    total_gross = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneyDraftSchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneyDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalGross",
    )
    tax_portions = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxPortionDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="taxPortions",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxedPriceDraft(**data)


class TaxedPriceSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxedPrice`."""

    total_net = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalNet",
    )
    total_gross = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalGross",
    )
    tax_portions = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxPortionSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="taxPortions",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxedPrice(**data)


class CartAddCustomLineItemActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddCustomLineItemAction`."""

    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    name = LocalizedStringField(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True)
    slug = marshmallow.fields.String(allow_none=True)
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddCustomLineItemAction(**data)


class CartAddDiscountCodeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddDiscountCodeAction`."""

    code = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddDiscountCodeAction(**data)


class CartAddItemShippingAddressActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddItemShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddItemShippingAddressAction(**data)


class CartAddLineItemActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddLineItemAction`."""

    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )
    product_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="productId"
    )
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )
    sku = marshmallow.fields.String(allow_none=True, missing=None)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddLineItemAction(**data)


class CartAddPaymentActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddPaymentAction`."""

    payment = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddPaymentAction(**data)


class CartAddShoppingListActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartAddShoppingListAction`."""

    shopping_list = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shoppingList",
    )
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartAddShoppingListAction(**data)


class CartApplyDeltaToCustomLineItemShippingDetailsTargetsActionSchema(
    CartUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.CartApplyDeltaToCustomLineItemShippingDetailsTargetsAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    targets_delta = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingTargetSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="targetsDelta",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartApplyDeltaToCustomLineItemShippingDetailsTargetsAction(**data)


class CartApplyDeltaToLineItemShippingDetailsTargetsActionSchema(
    CartUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.CartApplyDeltaToLineItemShippingDetailsTargetsAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    targets_delta = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingTargetSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="targetsDelta",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartApplyDeltaToLineItemShippingDetailsTargetsAction(**data)


class CartChangeCustomLineItemMoneyActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeCustomLineItemMoneyAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeCustomLineItemMoneyAction(**data)


class CartChangeCustomLineItemQuantityActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeCustomLineItemQuantityAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeCustomLineItemQuantityAction(**data)


class CartChangeLineItemQuantityActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeLineItemQuantityAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True)
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeLineItemQuantityAction(**data)


class CartChangeTaxCalculationModeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeTaxCalculationModeAction`."""

    tax_calculation_mode = marshmallow_enum.EnumField(
        types.TaxCalculationMode, by_value=True, data_key="taxCalculationMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeTaxCalculationModeAction(**data)


class CartChangeTaxModeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeTaxModeAction`."""

    tax_mode = marshmallow_enum.EnumField(
        types.TaxMode, by_value=True, data_key="taxMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeTaxModeAction(**data)


class CartChangeTaxRoundingModeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartChangeTaxRoundingModeAction`."""

    tax_rounding_mode = marshmallow_enum.EnumField(
        types.RoundingMode, by_value=True, data_key="taxRoundingMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartChangeTaxRoundingModeAction(**data)


class CartRecalculateActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRecalculateAction`."""

    update_product_data = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="updateProductData"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRecalculateAction(**data)


class CartRemoveCustomLineItemActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRemoveCustomLineItemAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRemoveCustomLineItemAction(**data)


class CartRemoveDiscountCodeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRemoveDiscountCodeAction`."""

    discount_code = helpers.LazyNestedField(
        nested="commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="discountCode",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRemoveDiscountCodeAction(**data)


class CartRemoveItemShippingAddressActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRemoveItemShippingAddressAction`."""

    address_key = marshmallow.fields.String(allow_none=True, data_key="addressKey")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRemoveItemShippingAddressAction(**data)


class CartRemoveLineItemActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRemoveLineItemAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )
    shipping_details_to_remove = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetailsToRemove",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRemoveLineItemAction(**data)


class CartRemovePaymentActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartRemovePaymentAction`."""

    payment = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartRemovePaymentAction(**data)


class CartSetAnonymousIdActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetAnonymousIdAction`."""

    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetAnonymousIdAction(**data)


class CartSetBillingAddressActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetBillingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetBillingAddressAction(**data)


class CartSetCartTotalTaxActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCartTotalTaxAction`."""

    external_total_gross = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="externalTotalGross",
    )
    external_tax_portions = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxPortionDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="externalTaxPortions",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCartTotalTaxAction(**data)


class CartSetCountryActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCountryAction`."""

    country = marshmallow.fields.String(missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCountryAction(**data)


class CartSetCustomFieldActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomFieldAction(**data)


class CartSetCustomLineItemCustomFieldActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomLineItemCustomFieldAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomLineItemCustomFieldAction(**data)


class CartSetCustomLineItemCustomTypeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomLineItemCustomTypeAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomLineItemCustomTypeAction(**data)


class CartSetCustomLineItemShippingDetailsActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomLineItemShippingDetailsAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomLineItemShippingDetailsAction(**data)


class CartSetCustomLineItemTaxAmountActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomLineItemTaxAmountAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomLineItemTaxAmountAction(**data)


class CartSetCustomLineItemTaxRateActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomLineItemTaxRateAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomLineItemTaxRateAction(**data)


class CartSetCustomShippingMethodActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomShippingMethodAction`."""

    shipping_method_name = marshmallow.fields.String(
        allow_none=True, data_key="shippingMethodName"
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomShippingMethodAction(**data)


class CartSetCustomTypeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomTypeAction(**data)


class CartSetCustomerEmailActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomerEmailAction`."""

    email = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomerEmailAction(**data)


class CartSetCustomerGroupActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomerGroupAction`."""

    customer_group = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="customerGroup",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomerGroupAction(**data)


class CartSetCustomerIdActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetCustomerIdAction`."""

    customer_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetCustomerIdAction(**data)


class CartSetDeleteDaysAfterLastModificationActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetDeleteDaysAfterLastModificationAction`."""

    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetDeleteDaysAfterLastModificationAction(**data)


class CartSetLineItemCustomFieldActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemCustomFieldAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemCustomFieldAction(**data)


class CartSetLineItemCustomTypeActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemCustomTypeAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemCustomTypeAction(**data)


class CartSetLineItemDistributionChannelActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemDistributionChannelAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemDistributionChannelAction(**data)


class CartSetLineItemPriceActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemPriceAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemPriceAction(**data)


class CartSetLineItemShippingDetailsActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemShippingDetailsAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemShippingDetailsAction(**data)


class CartSetLineItemTaxAmountActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemTaxAmountAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemTaxAmountAction(**data)


class CartSetLineItemTaxRateActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemTaxRateAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemTaxRateAction(**data)


class CartSetLineItemTotalPriceActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLineItemTotalPriceAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLineItemTotalPriceAction(**data)


class CartSetLocaleActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetLocaleAction`."""

    locale = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetLocaleAction(**data)


class CartSetShippingAddressActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetShippingAddressAction(**data)


class CartSetShippingMethodActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetShippingMethodAction`."""

    shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingMethod",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetShippingMethodAction(**data)


class CartSetShippingMethodTaxAmountActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetShippingMethodTaxAmountAction`."""

    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetShippingMethodTaxAmountAction(**data)


class CartSetShippingMethodTaxRateActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetShippingMethodTaxRateAction`."""

    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetShippingMethodTaxRateAction(**data)


class CartSetShippingRateInputActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartSetShippingRateInputAction`."""

    shipping_rate_input = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "Classification": "commercetools._schemas._cart.ClassificationShippingRateInputDraftSchema",
            "Score": "commercetools._schemas._cart.ScoreShippingRateInputDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingRateInput",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartSetShippingRateInputAction(**data)


class CartUpdateItemShippingAddressActionSchema(CartUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartUpdateItemShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartUpdateItemShippingAddressAction(**data)


class ClassificationShippingRateInputDraftSchema(ShippingRateInputDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.ClassificationShippingRateInputDraft`."""

    key = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ClassificationShippingRateInputDraft(**data)


class ClassificationShippingRateInputSchema(ShippingRateInputSchema):
    """Marshmallow schema for :class:`commercetools.types.ClassificationShippingRateInput`."""

    key = marshmallow.fields.String(allow_none=True)
    label = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ClassificationShippingRateInput(**data)


class ScoreShippingRateInputDraftSchema(ShippingRateInputDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.ScoreShippingRateInputDraft`."""

    score = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ScoreShippingRateInputDraft(**data)


class ScoreShippingRateInputSchema(ShippingRateInputSchema):
    """Marshmallow schema for :class:`commercetools.types.ScoreShippingRateInput`."""

    score = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ScoreShippingRateInput(**data)



import datetime
import random
import string
import typing
import uuid

from commercetools import types
from commercetools._schemas._tax_category import (
    TaxCategoryDraftSchema,
    TaxCategoryPagedQueryResponseSchema,
    TaxCategorySchema,
    TaxCategoryUpdateSchema,
    TaxRateSchema,
)
from commercetools.testing.abstract import BaseModel, ServiceBackend
from commercetools.testing.utils import (
    update_attribute,
    update_attribute_add_item,
    update_attribute_delete_item_by_id,
)


def generate_tax_rate_id():
    return "".join(random.choices(string.ascii_uppercase + string.digits, k=6))


class TaxCategoryModel(BaseModel):
    _primary_type_name = "tax-category"
    _resource_schema = TaxCategorySchema
    _unique_values = ["key"]

    def _create_from_draft(
        self, draft: types.TaxCategoryDraft, id: typing.Optional[str] = None
    ) -> types.TaxCategory:
        object_id = str(uuid.UUID(id) if id is not None else uuid.uuid4())
        return types.TaxCategory(
            id=str(object_id),
            key=draft.key,
            version=1,
            created_at=datetime.datetime.now(datetime.timezone.utc),
            last_modified_at=datetime.datetime.now(datetime.timezone.utc),
            name=draft.name,
            description=draft.description,
            rates=self._create_rates(draft.rates),
        )

    def _create_rates(
        self, drafts: typing.Optional[typing.List[types.TaxRateDraft]]
    ) -> typing.List[types.TaxRate]:
        result: typing.List[types.TaxRate] = []
        if not drafts:
            return result
        for draft in drafts:
            obj = types.TaxRate(
                id=generate_tax_rate_id(),
                name=draft.name,
                amount=draft.amount,
                included_in_price=draft.included_in_price,
                country=draft.country,
                state=draft.state,
                sub_rates=draft.sub_rates,
            )
            result.append(obj)
        return result


def add_tax_rate_action():
    return update_attribute_add_item(
        "rates", "tax_rate", TaxRateSchema, generate_tax_rate_id
    )


def replace_tax_rate_action():
    delete_action = update_attribute_delete_item_by_id("rates", "tax_rate_id")
    add_action = add_tax_rate_action()

    def updater(self, obj, action):
        obj = delete_action(self, obj, action)
        return add_action(self, obj, action)

    return updater


class TaxCategoryBackend(ServiceBackend):
    service_path = "tax-categories"
    model_class = TaxCategoryModel
    _schema_draft = TaxCategoryDraftSchema
    _schema_update = TaxCategoryUpdateSchema
    _schema_query_response = TaxCategoryPagedQueryResponseSchema

    def urls(self):
        return [
            ("^$", "GET", self.query),
            ("^$", "POST", self.create),
            ("^key=(?P<key>[^/]+)$", "GET", self.get_by_key),
            ("^(?P<id>[^/]+)$", "GET", self.get_by_id),
            ("^key=(?P<key>[^/]+)$", "POST", self.update_by_key),
            ("^(?P<id>[^/]+)$", "POST", self.update_by_id),
            ("^key=(?P<key>[^/]+)$", "DELETE", self.delete_by_key),
            ("^(?P<id>[^/]+)$", "DELETE", self.delete_by_id),
        ]

    _actions = {
        "changeName": update_attribute("name", "name"),
        "setKey": update_attribute("key", "key"),
        "setDescription": update_attribute("description", "description"),
        "addTaxRate": add_tax_rate_action(),
        "removeTaxRate": update_attribute_delete_item_by_id("rates", "tax_rate_id"),
        "replaceTaxRate": replace_tax_rate_action(),
    }



from commercetools import types


def test_product_discount_get_by_id(client):
    product_discount = client.product_discounts.create(
        types.ProductDiscountDraft(
            name=types.LocalizedString(nl="test-discount"),
            predicate="",
            value=types.ProductDiscountValueRelativeDraft(permyriad=10),
            is_active=True,
            sort_order="",
        )
    )

    assert product_discount.id



# DO NOT EDIT! This file is automatically generated

import datetime
import typing

from commercetools.types._abstract import _BaseType

if typing.TYPE_CHECKING:
    from ._common import Resource
__all__ = ["PagedQueryResponse", "Update", "UpdateAction"]


class PagedQueryResponse(_BaseType):
    "Corresponding marshmallow schema is :class:`commercetools.schemas.PagedQueryResponseSchema`."
    #: :class:`int`
    count: typing.Optional[int]
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: typing.Optional[int]
    #: List of :class:`commercetools.types.Resource`
    results: typing.Optional[typing.Sequence["Resource"]]

    def __init__(
        self,
        *,
        count: typing.Optional[int] = None,
        total: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        results: typing.Optional[typing.Sequence["Resource"]] = None
    ) -> None:
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return "PagedQueryResponse(count=%r, total=%r, offset=%r, results=%r)" % (
            self.count,
            self.total,
            self.offset,
            self.results,
        )


class Update(_BaseType):
    "Corresponding marshmallow schema is :class:`commercetools.schemas.UpdateSchema`."
    #: :class:`int`
    version: typing.Optional[int]
    #: :class:`list`
    actions: typing.Optional[list]

    def __init__(
        self,
        *,
        version: typing.Optional[int] = None,
        actions: typing.Optional[list] = None
    ) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "Update(version=%r, actions=%r)" % (self.version, self.actions)


class UpdateAction(_BaseType):
    "Corresponding marshmallow schema is :class:`commercetools.schemas.UpdateActionSchema`."
    #: :class:`str`
    action: typing.Optional[str]

    def __init__(self, *, action: typing.Optional[str] = None) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "UpdateAction(action=%r)" % (self.action,)
